﻿module Degenz.DbService

open Degenz.Types

open System
//open MongoDB.Bson
//open MongoDB.Driver
open Npgsql.FSharp

type User = {
    Name : string
    DiscordId : uint64
    Bank : int<GBT>
    Strength : int
    Inventory : int array
}

let mapBack user : PlayerData =
    { DiscordId = user.DiscordId
      Name = user.Name
      Inventory = user.Inventory |> Array.choose (fun id -> Armory.battleItems |> Array.tryFind (fun i -> i.Id = id))
      Events = [||]
      Traits = { PlayerTraits.empty with Strength = user.Strength }
//      Achievements =
//          tryWithDefault bson "Achievements" [||] (fun v ->
//              v.AsBsonArray |> Seq.map (fun (bv : BsonValue) -> bv.AsString) |> Seq.toArray)
//      XP = tryWithDefault bson "XP" 0 (fun v -> v.AsInt32)
      Bank = user.Bank
}

let tryFindPlayer connStr (discordId : uint64) =
    async {
        try
            let! user =
                connStr
                |> Sql.connect
                |> Sql.query ("SELECT discord_id, display_name, gbt, strength, inventory " +
                              "FROM \"user\" WHERE discord_id = @did")
                |> Sql.parameters [ "did", Sql.string (string discordId) ]
                |> Sql.executeAsync (fun read ->
                    {
                        DiscordId = read.string "discord_id" |> uint64
                        Name = read.string "display_name"
                        Bank = read.int "gbt" * 1<GBT>
                        Strength = read.int "strength"
                        Inventory = read.intArray "inventory"
                    })
                |> Async.AwaitTask

            match List.tryHead user with
            | None -> return None
            | Some u -> return Some (mapBack u)
        with e ->
            printfn $"{e.Message}"
            return None
    }

let updatePlayer connStr (player : PlayerData) =
    connStr
    |> Sql.connect
    |> Sql.query
        ("UPDATE \"user\" SET gbt = @gbt, strength = @str, inventory = @inv " +
        " WHERE discord_id = @did")
    |> Sql.parameters [
        "did", Sql.string (string player.DiscordId)
        "gbt", Sql.int (int player.Bank)
        "str", Sql.int (int player.Traits.Strength)
        "inv", Sql.intArray (player.Inventory |> Array.map (fun i -> i.Id))
    ]
    |> Sql.executeNonQueryAsync
    |> Async.AwaitTask

let addAchievement connStr (did : uint64) (achievement : string) =
    connStr
    |> Sql.connect
    |> Sql.query (
        "WITH ach AS (INSERT INTO achievement (symbol) VALUES (@symbol) RETURNING id),
             usr AS (SELECT id FROM \"user\" WHERE discord_id = @did)
        INSERT INTO user_achievements_achievement (user_id, achievement_id) SELECT usr.id, ach.id FROM usr, ach")

    |> Sql.parameters [ ( "did" , Sql.string (string did) ) ; ( "achievement", Sql.string achievement ) ]
    |> Sql.executeNonQueryAsync
    |> Async.AwaitTask
